/*------------------------------------------------------------------------------
 *       Filename:  prompter.h
 *    Description:  Header file for prompter program
 *        Version:  1.1
 *        Created:  08/01/2012
 *       Compiler:  gcc
 *        Authors:  Larry Holder, holder@eecs.wsu.edu
 *                  Narayanan C Krishnan, ckn@eecs.wsu.edu
 *                  Diane J. Cook, cook@eecs.wsu.edu
 *   Organization:  Washington State University
------------------------------------------------------------------------------*/

#ifndef PROMPTER_H
#define PROMPTER_H

#include <stdio.h>
#include <time.h>
#include <string>
#include <map>
#include "casas.h"

using namespace std;

// CASAS middleware parameters
const string serverName = "casas-lin";
const string activityRecognitionChannel = "ar";
const string chimeChannel = "chime";
const string promptChannel = "prompt";
const string prompterUserName = "prompt";
const string prompterPassword = "prompt";
const string prompterManager = "";

// Different modes of prompter
enum { LEARN_MODE, MONITOR_MODE, GENERATE_MODE, TEST_MODE };

// Features for start and end times of activity in seconds since epoch
enum { FEATURE_START_TIME, FEATURE_END_TIME, NUM_FEATURES };

// Special prompter activity names. Must be in same order as prompterActivityNames below
// ORDER IS IMPORTANT! Do not rearrange or insert anything.
enum { PROMPTER_ACTIVITY_YEAR, PROMPTER_ACTIVITY_MONTH, PROMPTER_ACTIVITY_WEEK, PROMPTER_ACTIVITY_DAY, PROMPTER_ACTIVITY_HOUR,
	   PROMPTER_ACTIVITY_JANUARY, PROMPTER_ACTIVITY_FEBRUARY, PROMPTER_ACTIVITY_MARCH, PROMPTER_ACTIVITY_APRIL,
	   PROMPTER_ACTIVITY_MAY, PROMPTER_ACTIVITY_JUNE, PROMPTER_ACTIVITY_JULY, PROMPTER_ACTIVITY_AUGUST,
	   PROMPTER_ACTIVITY_SEPTEMBER, PROMPTER_ACTIVITY_OCTOBER, PROMPTER_ACTIVITY_NOVEMBER, PROMPTER_ACTIVITY_DECEMBER,
	   PROMPTER_ACTIVITY_SUNDAY, PROMPTER_ACTIVITY_MONDAY, PROMPTER_ACTIVITY_TUESDAY, PROMPTER_ACTIVITY_WEDNESDAY,
	   PROMPTER_ACTIVITY_THURSDAY, PROMPTER_ACTIVITY_FRIDAY, PROMPTER_ACTIVITY_SATURDAY,
	   PROMPTER_ACTIVITY_HOUR_00, PROMPTER_ACTIVITY_HOUR_01, PROMPTER_ACTIVITY_HOUR_02, PROMPTER_ACTIVITY_HOUR_03,
	   PROMPTER_ACTIVITY_HOUR_04, PROMPTER_ACTIVITY_HOUR_05, PROMPTER_ACTIVITY_HOUR_06, PROMPTER_ACTIVITY_HOUR_07,
	   PROMPTER_ACTIVITY_HOUR_08, PROMPTER_ACTIVITY_HOUR_09, PROMPTER_ACTIVITY_HOUR_10, PROMPTER_ACTIVITY_HOUR_11,
	   PROMPTER_ACTIVITY_HOUR_12, PROMPTER_ACTIVITY_HOUR_13, PROMPTER_ACTIVITY_HOUR_14, PROMPTER_ACTIVITY_HOUR_15,
	   PROMPTER_ACTIVITY_HOUR_16, PROMPTER_ACTIVITY_HOUR_17, PROMPTER_ACTIVITY_HOUR_18, PROMPTER_ACTIVITY_HOUR_19,
	   PROMPTER_ACTIVITY_HOUR_20, PROMPTER_ACTIVITY_HOUR_21, PROMPTER_ACTIVITY_HOUR_22, PROMPTER_ACTIVITY_HOUR_23,
	   NUM_PROMPTER_ACTIVITIES };

// Months of the year
enum { MONTH_JANUARY = 1, MONTH_FEBRUARY, MONTH_MARCH, MONTH_APRIL, MONTH_MAY, MONTH_JUNE,
	   MONTH_JULY, MONTH_AUGUST, MONTH_SEPTEMBER, MONTH_OCTOBER, MONTH_NOVEMBER, MONTH_DECEMBER };

// Days of the week
enum { DAY_SUNDAY, DAY_MONDAY, DAY_TUESDAY, DAY_WEDNESDAY, DAY_THURSDAY, DAY_FRIDAY, DAY_SATURDAY };

#define MAX_LINE_LEN 40000
#define MAXSTR 80
#define TRUE 1
#define FALSE 0

// Activity pattern modes
enum { PATTERN_MODE_INACTIVE, PATTERN_MODE_ACTIVE, PATTERN_MODE_ONE_PROMPT, PATTERN_MODE_TWO_PROMPTS };

struct ActivityPattern
{
	int relativeActivity;
	float pValue;
	int mean;
	int stddev;
	int mode;
	time_t time;
};

struct ActivityStats
{
	int count;
	int error;
	int truePositives;
	int falsePositives;
	int falseNegatives;
	// there are no true negatives in the prompting scenario
};

// Must be in same order as PROMPTER_ACTIVITY enum above
char prompterActivityNames[NUM_PROMPTER_ACTIVITIES][MAXSTR] = {
		"Prompter_Activity_Year", "Prompter_Activity_Month", "Prompter_Activity_Week", "Prompter_Activity_Day", "Prompter_Activity_Hour",
		"Prompter_Activity_January", "Prompter_Activity_February", "Prompter_Activity_March", "Prompter_Activity_April",
		"Prompter_Activity_May", "Prompter_Activity_June", "Prompter_Activity_July", "Prompter_Activity_August",
		"Prompter_Activity_September", "Prompter_Activity_October","Prompter_Activity_November", "Prompter_Activity_December",
		"Prompter_Activity_Sunday", "Prompter_Activity_Monday",	"Prompter_Activity_Tuesday", "Prompter_Activity_Wednesday",
		"Prompter_Activity_Thursday", "Prompter_Activity_Friday", "Prompter_Activity_Saturday",
		"Prompter_Activity_Hour_00", "Prompter_Activity_Hour_01", "Prompter_Activity_Hour_02", "Prompter_Activity_Hour_03",
		"Prompter_Activity_Hour_04", "Prompter_Activity_Hour_05", "Prompter_Activity_Hour_06", "Prompter_Activity_Hour_07",
		"Prompter_Activity_Hour_08", "Prompter_Activity_Hour_09", "Prompter_Activity_Hour_10", "Prompter_Activity_Hour_11",
		"Prompter_Activity_Hour_12", "Prompter_Activity_Hour_13", "Prompter_Activity_Hour_14", "Prompter_Activity_Hour_15",
		"Prompter_Activity_Hour_16", "Prompter_Activity_Hour_17", "Prompter_Activity_Hour_18", "Prompter_Activity_Hour_19",
		"Prompter_Activity_Hour_20", "Prompter_Activity_Hour_21", "Prompter_Activity_Hour_22", "Prompter_Activity_Hour_23"
};

int OutputLevel = 0;				// 0 = some output; 1 = more output; 2 = much output
Casas *cbot = NULL;					// Interface to CASAS middleware
int prompterMode = 0;				// Operating mode of prompter
char **filenames = NULL;			// names of the training data files
char *patternfname = NULL;			// filename for storing / loading activity patterns
char **sensornames = NULL;			// sensor names
char **activitynames = NULL;		// activity names
char **promptactivitynames = NULL;	// prompt activity names
int NumFiles = 0;					// number of training data files
int NumActivities = 0;				// total number of activities
int NumPromptActivities = 0;		// total number of prompt activities
int NumSensors = 0;					// total number of sensors
int **adindices = NULL;				// indices of samples for each activity.
int *numdataperactivity = NULL;		// number of samples per activity
int NumData = 0;					// total number of data samples
int **data = NULL;					// feature vector matrix derived from sensor data
time_t DataStartTime = 0;
time_t DataEndTime = 0;
ActivityPattern** ActivityPatterns = NULL;
int *NumPatternsPerActivity = NULL;
ActivityStats *ActivityStatistics = NULL;

// Prototypes

void ReadConfig (char *);
void AddPrompterActivities();
void AddPrompterEvents();
void AddBeforeAfterPrompterEvents (time_t, bool);
void PrintParms();
void PrompterUsage();
void PrintActivityInstances (int, int);
void PrintFeatures (int);
void CheckPromptActivityPatterns();
void ReadActivityPatterns();
void WriteActivityPatterns();
void PrintActivityPatterns (FILE*);
void LearnActivityPatterns();
int ComputeActivityStatistics (int, int, int, int*, int*);
bool ConsiderSemiPeriodicPatterns (int, float);
void ReadData();
int FindActivity (char *);
int FindSensor (char *);
void AddData (int, int*);
void AddDataToActivity (int, int);
void SetFeatures (int *, int, int);
void SetStartTime (char *, char *, int *);
void SetEndTime (char *, char *, int *);
time_t GetStartTime (int *);
time_t GetEndTime (int *);
void AllocateMemory();
void FreeMemory();

void InitMonitorInterface();
void HandleAR (map<string, string>);
void HandlePrompt (map<string, string>);
void HandleChime (map<string, string>);
void PrintPacket (map<string, string>);
void CheckPrompts();
void IssuePrompt (int, int);
void UpdatePatternsByActivity (int, time_t);

void GenerateData (time_t, time_t);
void GenerateActivity (int, time_t);
void WriteActivity (FILE *, int, time_t);
int SampleNormal (int, int);

void TestData();
void EvaluatePatternsByActivity (int, time_t);

#endif // PROMPTER_H
